	# firefox.py
# An app module for NVDA
# Author: paulber19<paulber19@laposte.net>
# Copyright 2016, released under GPL.
#See the file COPYING for more details.

import addonHandler
addonHandler.initTranslation()
from logHandler import log
import os
import appModuleHandler
import api
import controlTypes
import wx
from NVDAObjects.IAccessible import IAccessible
from NVDAObjects import NVDAObject
from firefoxPack.configManager import _addonConfigManager 
from firefoxPack.ia2 import * 
import speech
import ui
import gui
from gui.settingsDialogs import SettingsDialog
from keyboardHandler import KeyboardInputGesture
import config
import winUser
from oleacc import *

class StatusBar(object):
	def __init__(self):
		self.obj = None
		oForeground = api.getForegroundObject()
		oForeground.ia2ID = get_ia2ID(oForeground)
		self.mainWindow = None
		if oForeground.ia2ID == "main-window":
			self.mainWindow = oForeground
			self.obj = self._getStatusBar()
	
	@classmethod
	def getStatusBar(self):
		return  StatusBar().obj
	
	def searchForStatusBar(self, obj):
		o = obj.firstChild
		while o:
			ia2ID = get_ia2ID(o)
			if o.role == controlTypes.ROLE_PROPERTYPAGE and ia2ID and"panel-" in ia2ID  and o.lastChild.role == controlTypes.ROLE_STATUSBAR:
				return o
			o = o.next
		return None
	
	
	def searchForStatusBarIAObject(self, obj):
		(o, id)  = accNavigate(obj ,0,NAVDIR_FIRSTCHILD)
		while o :
			id = ""
			attributes = splitIA2Attribs(o.attributes)
			if (attributes and "id" in attributes.keys()
				and o.accRole(0) == ROLE_SYSTEM_PROPERTYPAGE and "panel-" in attributes["id"]):
				(oLast, id)  = accNavigate(o ,0,NAVDIR_LASTCHILD)
				if oLast and oLast.accRole(0)  == ROLE_SYSTEM_STATUSBAR:
					return oLast
			
			try:
				(o, id) = accNavigate(o,0,NAVDIR_NEXT)
			except:
				o = None
		return None
	
	def _getStatusBar(self):
		o = self.mainWindow.lastChild
		while o:
			if controlTypes.STATE_INVISIBLE not in o.states and o.role == controlTypes.ROLE_GROUPING :
				oStatusBar = self.searchForStatusBar(o)
				if oStatusBar is not None:
					return oStatusBar
			
			o = o.previous
		
		return None
	
	def _getStatusBarIAObject(self):
		(o, id)  = accNavigate(self.mainWindow.IAccessibleObject ,0,NAVDIR_LASTCHILD)
		while o:
			if o.accRole(0) == ROLE_SYSTEM_GROUPING and not o.accState(0) &STATE_SYSTEM_INVISIBLE :
				oStatusBar = self.searchForStatusBarIAObject(o)
				if oStatusBar is not None:
					return oStatusBar
			
			try:
				(o, id) = accNavigate(o,0,NAVDIR_PREVIOUS)
			except:
				o = None
		
		return None
	
	def getStatusBarText(self):
		o= self._getStatusBarIAObject()
		(o, id)  = accNavigate(o ,0,NAVDIR_FIRSTCHILD)
		if o and o.accName(0):
			return o.accName(0)
			
		return None
	

class TabsToolBar(object):
	def __init__(self):
		self.oTabs= None
		oForeground = api.getForegroundObject()
		oForeground.ia2ID = get_ia2ID(oForeground)
		if oForeground.ia2ID  and oForeground.ia2ID != "main-window":
			# not in main window
			return
		oTabsToolbar = findObjectByIA2IdEx(oForeground, "TabsToolbar")
		if oTabsToolbar is None:
			# tabs toolbar not found
			log.warning("tabs toolbar not found in main Window")
			return
			
		self.oTabs = oTabsToolbar.firstChild
		self.oTabsToolbar = oTabsToolbar
	def setFocus(self):
		self.getSelectedTab().IAccessibleObject.accSelect(1,0)


	def getSelectedTab(self):
		index = self.getSelectedTabIndex()
		return self.oTabs.getChild(index)
	
	def getSelectedTabIndex(self):
		i=0
		(o, id)  = accNavigate(self.oTabs.IAccessibleObject ,0,NAVDIR_FIRSTCHILD)
		while o:
			if o.accState(0) & STATE_SYSTEM_SELECTED:
				return i
				
			try:
				(o, id) = accNavigate(o,0,NAVDIR_NEXT)
				i= i+1
			except:
				o= None
	
		return None
	
	
	def getTabPosition(self):
		if self.oTabs is None:
			return""
		
		count = self.oTabs.childCount -1
		if count>1:
			index = self.getSelectedTabIndex()
			return _("Tab {index} of {count}").format(index = index+1, count = count)
		
		return ""
	
	def switchTo (self, oTab, speechMode):
		api.processPendingEvents()
		speech.cancelSpeech()
		speech.speechMode = speechMode
		oTab.doAction()

class Document(NVDAObject):

	def _get_name(self):
		name=  super(Document, self)._get_name()
		if name is not None:
			name = "%s %s" %(name, TabsToolBar().getTabPosition())
		return name
	
	def speakStatusBarText(self):
		if self.role == controlTypes.ROLE_DOCUMENT and  controlTypes.STATE_BUSY in self.states and winUser.isWindowVisible(self.windowHandle) and self.isInForeground:
			statusBarText = StatusBar().getStatusBarText() if _addonConfigManager .toggleDynamicalyStatusLineReportOption(False) else None
			if statusBarText:
				speech.cancelSpeech()
				speech.speakMessage(controlTypes.stateLabels[controlTypes.STATE_BUSY])
				speech.speakMessage(statusBarText)
				return True
		return False
	
	def event_stateChange(self):
		if self.speakStatusBarText():
			return
		super(Document, self).event_stateChange()
	
	
	def event_gainFocus (self):
		if self.speakStatusBarText():
			return
		super(Document, self).event_gainFocus()

class Tab(NVDAObject):
	def initOverlayClass(self):
		self.bindGesture("kb:enter", "enter")
		
	def script_enter(self, gesture):
			KeyboardInputGesture.fromName("f6").send()
	
	__gestures ={
		"kb:enter": "enter",
		}

class AppModule(appModuleHandler.AppModule):
	_confspec = {
		"dynamicalyStatusLineReport": "boolean(default=False)",
		}
	
	def __init__(self, *args, **kwargs):
		super(AppModule,  self).__init__(*args, **kwargs)
		self.installSettingsMenu()
		
	
	def installSettingsMenu(self):
		self.prefsMenu = gui.mainFrame.sysTrayIcon.preferencesMenu
		self.firefoxMenu = self.prefsMenu.Append(wx.ID_ANY,
			# Translators: name of the option in the menu.
			_("Firefox appModule's  settings..."),
			"")
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onFirefoxMenu, self.firefoxMenu)
	
	def terminate (self):
		for item in [self.firefoxMenu, ]:
			try:
				self.prefsMenu.RemoveItem(item)
			except:
				pass
	
	def onFirefoxMenu(self, evt):
		gui.mainFrame._popupSettingsDialog(FirefoxSettingsDialog)

	def chooseNVDAObjectOverlayClasses(self, obj, clsList):
		obj.ia2ID = get_ia2ID(obj)
		if (obj.ia2ID and obj.ia2ID == "main-window" 
		or obj.role == controlTypes.ROLE_DOCUMENT):
			clsList.insert(0, Document)
		
		# check if focus is a tab  in tabsToolbar.
		if obj.role == controlTypes.ROLE_TAB:
			oParent = obj.parent
			if (oParent.role == controlTypes.ROLE_TABCONTROL 
			and oParent.ia2ID == "tabbrowser-tabs"):
				clsList.insert(0, Tab)				
	
	def event_appModule_gainFocus(self):
		self.oldGetStatusBar = api.getStatusBar
		api.getStatusBar = StatusBar.getStatusBar
	
	def event_appModule_loseFocus(self):
		if hasattr(self, "oldGetStatusBar "):
			api.getStatusBar = self.oldGetStatusBar 
	
	def script_goToTabsToolbar(self, gesture):
		TabsToolBar().setFocus()
	
	script_goToTabsToolbar.__doc__ = _("Move the focus to tabs toolbar")
	script_goToTabsToolbar.category = _("Firefox")
	
	def script_tabMovement(self, gesture):
		gesture.send()
		time.sleep(0.3)
		obj=api.getForegroundObject()
		title=obj.name
		if not isinstance(title,basestring) or not title or title.isspace():
			title=obj.appModule.appName  if obj.appModule else None
			if not isinstance(title,basestring) or not title or title.isspace():
				return
		
		ui.message(title)
	
	__gestures ={
		"kb:control+=":"goToTabsToolbar",
		"kb:control+shift+pageUp": "tabMovement",
		"kb:control+shift+pageDown": "tabMovement",
		}

class FirefoxSettingsDialog(SettingsDialog):
	# Translators: This is the label for the firefox settings  dialog.
	title = _("Firefox appModule settings")

	def makeSettings(self, settingsSizer):
		sHelper = gui.guiHelper.BoxSizerHelper(self, sizer=settingsSizer)
		# Translators: This is the label for a checkbox in the FirefoxSettingsDialog.
		dynamicStatusLineReportLabelText = _("Report dinamicaly the status line")
		self.dynamicStatusLineReportBox=sHelper.addItem(wx.CheckBox(self,wx.NewId(),label=dynamicStatusLineReportLabelText ))
		self.dynamicStatusLineReportBox.SetValue(_addonConfigManager .toggleDynamicalyStatusLineReportOption(False))
		
	def postInit(self):
		self.dynamicStatusLineReportBox.SetFocus()
		
	def saveSettingChanges (self):
		if self.dynamicStatusLineReportBox.IsChecked() != _addonConfigManager .toggleDynamicalyStatusLineReportOption(False):
			_addonConfigManager .toggleDynamicalyStatusLineReportOption(True)
		
	def onOk(self,evt):
		super(FirefoxSettingsDialog, self).onOk(evt)
		self.saveSettingChanges()
